"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var _ = require("lodash");
var SdlPrinter = /** @class */ (function () {
    function SdlPrinter() {
    }
    SdlPrinter.prototype.print = function (tables) {
        return __awaiter(this, void 0, void 0, function () {
            var _this = this;
            var candidates, sdl;
            return __generator(this, function (_a) {
                candidates = tables.filter(function (x) { return !x.isJunctionTable; });
                sdl = _.map(candidates, function (table) {
                    return _this.printType(table, tables.filter(function (x) { return x != table; }));
                });
                return [2 /*return*/, sdl.join('\r\n')];
            });
        });
    };
    SdlPrinter.prototype.printType = function (table, otherTables) {
        var _this = this;
        var nativeFields = table.columns;
        var filterFunction = function (c) {
            return (c.relation || false) && c.relation.table == table.name;
        };
        var relationFields = otherTables
            .filter(function (t) { return t.columns.some(filterFunction); })
            .map(function (t) {
            return t.columns
                .filter(filterFunction)
                .map(function (c) { return ({ remoteColumn: c, remoteTable: t }); });
        })
            .reduce(function (acc, next) { return acc.concat(next); }, []);
        return "type " + this.capitalizeFirstLetter(table.name) + " @pgTable(name: \"" + table.name + "\") {" + _.map(nativeFields, function (column) { return _this.printField(column); }).join('') + relationFields
            .map(function (field) { return _this.printBackRelationField(field); })
            .join('') + "\n}\n";
    };
    SdlPrinter.prototype.printBackRelationField = function (field) {
        if (field.remoteTable.isJunctionTable) {
            var otherRemoteTableField = field.remoteTable.columns.filter(function (x) { return x.name !== field.remoteColumn.name; })[0];
            var relatedTable = otherRemoteTableField.relation.table;
            return "\n  " + this.lowerCaseFirstLetter(relatedTable) + "s: [" + this.capitalizeFirstLetter(relatedTable) + "] @pgRelationTable(table: \"" + field.remoteTable.name + "\" name: \"" + field.remoteTable.name + "\")";
        }
        else {
            return "\n  " + field.remoteTable.name + "s: [" + this.capitalizeFirstLetter(field.remoteTable.name) + "]";
        }
    };
    SdlPrinter.prototype.printField = function (column) {
        return "\n  " + this.printFieldName(column) + ": " + this.printFieldType(column) + this.printFieldOptional(column) + this.printFieldDirective(column);
    };
    SdlPrinter.prototype.printFieldName = function (column) {
        if (column.relation) {
            return this.removeIdSuffix(column.name);
        }
        else {
            return column.name;
        }
    };
    SdlPrinter.prototype.printFieldType = function (column) {
        if (column.relation) {
            return (this.capitalizeFirstLetter(column.relation.table) +
                (" @pgRelation(column: \"" + column.name + "\")"));
        }
        else {
            return column.typeIdentifier;
        }
    };
    SdlPrinter.prototype.printFieldOptional = function (column) {
        return column.nullable ? '' : '!';
    };
    SdlPrinter.prototype.printFieldDirective = function (column) {
        if (column.isUnique) {
            return " @pgUnique";
        }
        if (column.defaultValue != null) {
            if (column.defaultValue == '[AUTO INCREMENT]') {
                return '';
            }
            if (column.typeIdentifier == 'String' ||
                column.typeIdentifier == 'DateTime' ||
                column.typeIdentifier == 'Json') {
                return " @pgDefault(value = \"" + column.defaultValue + "\")";
            }
            else {
                return " @pgDefault(value = " + column.defaultValue + ")";
            }
        }
        return '';
    };
    SdlPrinter.prototype.capitalizeFirstLetter = function (string) {
        return string.charAt(0).toUpperCase() + string.slice(1);
    };
    SdlPrinter.prototype.lowerCaseFirstLetter = function (string) {
        return string.charAt(0).toLowerCase() + string.slice(1);
    };
    SdlPrinter.prototype.removeIdSuffix = function (string) {
        function removeSuffix(suffix, string) {
            if (string.endsWith(suffix)) {
                return string.substring(0, string.length - suffix.length);
            }
            else {
                return string;
            }
        }
        return removeSuffix('_ID', removeSuffix('_id', removeSuffix('Id', string)));
    };
    return SdlPrinter;
}());
exports.SdlPrinter = SdlPrinter;
//# sourceMappingURL=SdlPrinter.js.map